@@ -1,14 +1,14 @@
 # Tweepy
-# Copyright 2009 Joshua Roesslein
-# See LICENSE
+# Copyright 2009-2010 Joshua Roesslein
+# See LICENSE for details.
 
 import os
 import mimetypes
 
 from tweepy.binder import bind_api
 from tweepy.error import TweepError
-from tweepy.parsers import *
-from tweepy.models import ModelFactory
+from tweepy.parsers import ModelParser
+from tweepy.utils import list_to_csv
 
 
 class API(object):
@@ -18,7 +18,7 @@ class API(object):
             host='api.twitter.com', search_host='search.twitter.com',
              cache=None, secure=False, api_root='/1', search_root='',
             retry_count=0, retry_delay=0, retry_errors=None,
-            model_factory=None):
+            parser=None):
         self.auth = auth_handler
         self.host = host
         self.search_host = search_host
@@ -29,19 +29,12 @@ class API(object):
         self.retry_count = retry_count
         self.retry_delay = retry_delay
         self.retry_errors = retry_errors
-        self.model_factory = model_factory or ModelFactory
-
-    """ statuses/public_timeline """
-    public_timeline = bind_api(
-        path = '/statuses/public_timeline.json',
-        parser = parse_statuses,
-        allowed_param = []
-    )
+        self.parser = parser or ModelParser()
 
     """ statuses/home_timeline """
     home_timeline = bind_api(
         path = '/statuses/home_timeline.json',
-        parser = parse_statuses,
+        payload_type = 'status', payload_list = True,
         allowed_param = ['since_id', 'max_id', 'count', 'page'],
         require_auth = True
     )
@@ -49,7 +42,7 @@ class API(object):
     """ statuses/friends_timeline """
     friends_timeline = bind_api(
         path = '/statuses/friends_timeline.json',
-        parser = parse_statuses,
+        payload_type = 'status', payload_list = True,
         allowed_param = ['since_id', 'max_id', 'count', 'page'],
         require_auth = True
     )
@@ -57,23 +50,55 @@ class API(object):
     """ statuses/user_timeline """
     user_timeline = bind_api(
         path = '/statuses/user_timeline.json',
-        parser = parse_statuses,
+        payload_type = 'status', payload_list = True,
         allowed_param = ['id', 'user_id', 'screen_name', 'since_id',
-                          'max_id', 'count', 'page']
+                          'max_id', 'count', 'page', 'include_rts']
     )
 
     """ statuses/mentions """
     mentions = bind_api(
         path = '/statuses/mentions.json',
-        parser = parse_statuses,
+        payload_type = 'status', payload_list = True,
         allowed_param = ['since_id', 'max_id', 'count', 'page'],
         require_auth = True
     )
 
+    """/statuses/retweeted_by_user.format"""
+    retweeted_by_user = bind_api(
+        path = '/statuses/retweeted_by_user.json',
+        payload_type = 'status', payload_list = True,
+        allowed_param = ['screen_name', 'id', 'count', 'since_id',
+                            'max_id', 'page', 'include_entities']
+    )
+
+    """/statuses/:id/retweeted_by.format"""
+    retweeted_by = bind_api(
+        path = '/statuses/{id}/retweeted_by.json',
+        payload_type = 'status', payload_list = True,
+        allowed_param = ['id', 'count', 'page'],
+        require_auth = True
+    )
+
+    """/related_results/show/:id.format"""
+    related_results = bind_api(
+        path = '/related_results/show/{id}.json',
+        payload_type = 'relation', payload_list = True,
+        allowed_param = ['id'],
+        require_auth = False
+	)
+
+    """/statuses/:id/retweeted_by/ids.format"""
+    retweeted_by_ids = bind_api(
+        path = '/statuses/{id}/retweeted_by/ids.json',
+        payload_type = 'ids',
+        allowed_param = ['id', 'count', 'page'],
+        require_auth = True
+    )
+
     """ statuses/retweeted_by_me """
     retweeted_by_me = bind_api(
         path = '/statuses/retweeted_by_me.json',
-        parser = parse_statuses,
+        payload_type = 'status', payload_list = True,
         allowed_param = ['since_id', 'max_id', 'count', 'page'],
         require_auth = True
     )
@@ -81,7 +106,7 @@ class API(object):
     """ statuses/retweeted_to_me """
     retweeted_to_me = bind_api(
         path = '/statuses/retweeted_to_me.json',
-        parser = parse_statuses,
+        payload_type = 'status', payload_list = True,
         allowed_param = ['since_id', 'max_id', 'count', 'page'],
         require_auth = True
     )
@@ -89,7 +114,7 @@ class API(object):
     """ statuses/retweets_of_me """
     retweets_of_me = bind_api(
         path = '/statuses/retweets_of_me.json',
-        parser = parse_statuses,
+        payload_type = 'status', payload_list = True,
         allowed_param = ['since_id', 'max_id', 'count', 'page'],
         require_auth = True
     )
@@ -97,7 +122,7 @@ class API(object):
     """ statuses/show """
     get_status = bind_api(
         path = '/statuses/show.json',
-        parser = parse_status,
+        payload_type = 'status',
         allowed_param = ['id']
     )
 
@@ -105,8 +130,8 @@ class API(object):
     update_status = bind_api(
         path = '/statuses/update.json',
         method = 'POST',
-        parser = parse_status,
-        allowed_param = ['status', 'in_reply_to_status_id', 'lat', 'long', 'source'],
+        payload_type = 'status',
+        allowed_param = ['status', 'in_reply_to_status_id', 'lat', 'long', 'source', 'place_id'],
         require_auth = True
     )
 
@@ -114,7 +139,7 @@ class API(object):
     destroy_status = bind_api(
         path = '/statuses/destroy.json',
         method = 'DELETE',
-        parser = parse_status,
+        payload_type = 'status',
         allowed_param = ['id'],
         require_auth = True
     )
@@ -123,7 +148,7 @@ class API(object):
     retweet = bind_api(
         path = '/statuses/retweet/{id}.json',
         method = 'POST',
-        parser = parse_status,
+        payload_type = 'status',
         allowed_param = ['id'],
         require_auth = True
     )
@@ -131,7 +156,7 @@ class API(object):
     """ statuses/retweets """
     retweets = bind_api(
         path = '/statuses/retweets/{id}.json',
-        parser = parse_statuses,
+        payload_type = 'status', payload_list = True,
         allowed_param = ['id', 'count'],
         require_auth = True
     )
@@ -139,10 +164,20 @@ class API(object):
     """ users/show """
     get_user = bind_api(
         path = '/users/show.json',
-        parser = parse_user,
+        payload_type = 'user',
         allowed_param = ['id', 'user_id', 'screen_name']
     )
 
+    """ Perform bulk look up of users from user ID or screenname """
+    def lookup_users(self, user_ids=None, screen_names=None):
+        return self._lookup_users(list_to_csv(user_ids), list_to_csv(screen_names))
+
+    _lookup_users = bind_api(
+        path = '/users/lookup.json',
+        payload_type = 'user', payload_list = True,
+        allowed_param = ['user_id', 'screen_name'],
+    )
+
     """ Get the authenticated user """
     def me(self):
         return self.get_user(screen_name=self.auth.get_username())
@@ -150,7 +185,7 @@ class API(object):
     """ users/search """
     search_users = bind_api(
         path = '/users/search.json',
-        parser = parse_users,
+        payload_type = 'user', payload_list = True,
         require_auth = True,
         allowed_param = ['q', 'per_page', 'page']
     )
@@ -158,29 +193,37 @@ class API(object):
     """ statuses/friends """
     friends = bind_api(
         path = '/statuses/friends.json',
-        parser = parse_users,
+        payload_type = 'user', payload_list = True,
         allowed_param = ['id', 'user_id', 'screen_name', 'page', 'cursor']
     )
 
     """ statuses/followers """
     followers = bind_api(
         path = '/statuses/followers.json',
-        parser = parse_users,
+        payload_type = 'user', payload_list = True,
         allowed_param = ['id', 'user_id', 'screen_name', 'page', 'cursor']
     )
 
     """ direct_messages """
     direct_messages = bind_api(
         path = '/direct_messages.json',
-        parser = parse_directmessages,
+        payload_type = 'direct_message', payload_list = True,
         allowed_param = ['since_id', 'max_id', 'count', 'page'],
         require_auth = True
     )
 
+    """ direct_messages/show """
+    get_direct_message = bind_api(
+        path = '/direct_messages/show/{id}.json',
+        payload_type = 'direct_message',
+        allowed_param = ['id'],
+        require_auth = True
+    )
+
     """ direct_messages/sent """
     sent_direct_messages = bind_api(
         path = '/direct_messages/sent.json',
-        parser = parse_directmessages,
+        payload_type = 'direct_message', payload_list = True,
         allowed_param = ['since_id', 'max_id', 'count', 'page'],
         require_auth = True
     )
@@ -189,7 +232,7 @@ class API(object):
     send_direct_message = bind_api(
         path = '/direct_messages/new.json',
         method = 'POST',
-        parser = parse_dm,
+        payload_type = 'direct_message',
         allowed_param = ['user', 'screen_name', 'user_id', 'text'],
         require_auth = True
     )
@@ -198,7 +241,7 @@ class API(object):
     destroy_direct_message = bind_api(
         path = '/direct_messages/destroy.json',
         method = 'DELETE',
-        parser = parse_dm,
+        payload_type = 'direct_message',
         allowed_param = ['id'],
         require_auth = True
     )
@@ -207,7 +250,7 @@ class API(object):
     create_friendship = bind_api(
         path = '/friendships/create.json',
         method = 'POST',
-        parser = parse_user,
+        payload_type = 'user',
         allowed_param = ['id', 'user_id', 'screen_name', 'follow'],
         require_auth = True
     )
@@ -216,7 +259,7 @@ class API(object):
     destroy_friendship = bind_api(
         path = '/friendships/destroy.json',
         method = 'DELETE',
-        parser = parse_user,
+        payload_type = 'user',
         allowed_param = ['id', 'user_id', 'screen_name'],
         require_auth = True
     )
@@ -224,14 +267,14 @@ class API(object):
     """ friendships/exists """
     exists_friendship = bind_api(
         path = '/friendships/exists.json',
-        parser = parse_json,
+        payload_type = 'json',
         allowed_param = ['user_a', 'user_b']
     )
 
     """ friendships/show """
     show_friendship = bind_api(
         path = '/friendships/show.json',
-        parser = parse_friendship,
+        payload_type = 'friendship',
         allowed_param = ['source_id', 'source_screen_name',
                           'target_id', 'target_screen_name']
     )
@@ -239,32 +282,50 @@ class API(object):
     """ friends/ids """
     friends_ids = bind_api(
         path = '/friends/ids.json',
-        parser = parse_ids,
+        payload_type = 'ids',
         allowed_param = ['id', 'user_id', 'screen_name', 'cursor']
     )
 
+    """ friendships/incoming """
+    friendships_incoming = bind_api(
+        path = '/friendships/incoming.json',
+        payload_type = 'ids',
+        allowed_param = ['cursor']
+    )
+
+    """ friendships/outgoing"""
+    friendships_outgoing = bind_api(
+        path = '/friendships/outgoing.json',
+        payload_type = 'ids',
+        allowed_param = ['cursor']
+    )
+
     """ followers/ids """
     followers_ids = bind_api(
         path = '/followers/ids.json',
-        parser = parse_ids,
+        payload_type = 'ids',
         allowed_param = ['id', 'user_id', 'screen_name', 'cursor']
     )
 
     """ account/verify_credentials """
-    def verify_credentials(self):
+    def verify_credentials(self, **kargs):
         try:
             return bind_api(
                 path = '/account/verify_credentials.json',
-                parser = parse_user,
-                require_auth = True
-            )(self)
-        except TweepError:
-            return False
+                payload_type = 'user',
+                require_auth = True,
+                allowed_param = ['include_entities', 'skip_status'],
+            )(self, **kargs)
+        except TweepError, e:
+            if e.response and e.response.status == 401:
+                return False
+            raise
 
     """ account/rate_limit_status """
     rate_limit_status = bind_api(
         path = '/account/rate_limit_status.json',
-        parser = parse_json
+        payload_type = 'json',
+        use_cache = False
     )
 
     """ account/update_delivery_device """
@@ -272,7 +333,7 @@ class API(object):
         path = '/account/update_delivery_device.json',
         method = 'POST',
         allowed_param = ['device'],
-        parser = parse_user,
+        payload_type = 'user',
         require_auth = True
     )
 
@@ -280,7 +341,7 @@ class API(object):
     update_profile_colors = bind_api(
         path = '/account/update_profile_colors.json',
         method = 'POST',
-        parser = parse_user,
+        payload_type = 'user',
         allowed_param = ['profile_background_color', 'profile_text_color',
                           'profile_link_color', 'profile_sidebar_fill_color',
                           'profile_sidebar_border_color'],
@@ -293,7 +354,7 @@ class API(object):
         return bind_api(
             path = '/account/update_profile_image.json',
             method = 'POST',
-            parser = parse_user,
+            payload_type = 'user',
             require_auth = True
         )(self, post_data=post_data, headers=headers)
 
@@ -303,7 +364,7 @@ class API(object):
         bind_api(
             path = '/account/update_profile_background_image.json',
             method = 'POST',
-            parser = parse_user,
+            payload_type = 'user',
             allowed_param = ['tile'],
             require_auth = True
         )(self, post_data=post_data, headers=headers)
@@ -312,7 +373,7 @@ class API(object):
     update_profile = bind_api(
         path = '/account/update_profile.json',
         method = 'POST',
-        parser = parse_user,
+        payload_type = 'user',
         allowed_param = ['name', 'url', 'location', 'description'],
         require_auth = True
     )
@@ -320,15 +381,15 @@ class API(object):
     """ favorites """
     favorites = bind_api(
         path = '/favorites.json',
-        parser = parse_statuses,
-        allowed_param = ['id', 'page']
+        payload_type = 'status', payload_list = True,
+        allowed_param = ['id', 'max_id', 'page']
     )
 
     """ favorites/create """
     create_favorite = bind_api(
         path = '/favorites/create/{id}.json',
         method = 'POST',
-        parser = parse_status,
+        payload_type = 'status',
         allowed_param = ['id'],
         require_auth = True
     )
@@ -337,7 +398,7 @@ class API(object):
     destroy_favorite = bind_api(
         path = '/favorites/destroy/{id}.json',
         method = 'DELETE',
-        parser = parse_status,
+        payload_type = 'status',
         allowed_param = ['id'],
         require_auth = True
     )
@@ -346,7 +407,7 @@ class API(object):
     enable_notifications = bind_api(
         path = '/notifications/follow.json',
         method = 'POST',
-        parser = parse_user,
+        payload_type = 'user',
         allowed_param = ['id', 'user_id', 'screen_name'],
         require_auth = True
     )
@@ -355,7 +416,7 @@ class API(object):
     disable_notifications = bind_api(
         path = '/notifications/leave.json',
         method = 'POST',
-        parser = parse_user,
+        payload_type = 'user',
         allowed_param = ['id', 'user_id', 'screen_name'],
         require_auth = True
     )
@@ -364,7 +425,7 @@ class API(object):
     create_block = bind_api(
         path = '/blocks/create.json',
         method = 'POST',
-        parser = parse_user,
+        payload_type = 'user',
         allowed_param = ['id', 'user_id', 'screen_name'],
         require_auth = True
     )
@@ -373,7 +434,7 @@ class API(object):
     destroy_block = bind_api(
         path = '/blocks/destroy.json',
         method = 'DELETE',
-        parser = parse_user,
+        payload_type = 'user',
         allowed_param = ['id', 'user_id', 'screen_name'],
         require_auth = True
     )
@@ -383,7 +444,6 @@ class API(object):
         try:
             bind_api(
                 path = '/blocks/exists.json',
-                parser = parse_none,
                 allowed_param = ['id', 'user_id', 'screen_name'],
                 require_auth = True
             )(self, *args, **kargs)
@@ -394,7 +454,7 @@ class API(object):
     """ blocks/blocking """
     blocks = bind_api(
         path = '/blocks/blocking.json',
-        parser = parse_users,
+        payload_type = 'user', payload_list = True,
         allowed_param = ['page'],
         require_auth = True
     )
@@ -402,7 +462,7 @@ class API(object):
     """ blocks/blocking/ids """
     blocks_ids = bind_api(
         path = '/blocks/blocking/ids.json',
-        parser = parse_json,
+        payload_type = 'json',
         require_auth = True
     )
 
@@ -410,7 +470,7 @@ class API(object):
     report_spam = bind_api(
         path = '/report_spam.json',
         method = 'POST',
-        parser = parse_user,
+        payload_type = 'user',
         allowed_param = ['id', 'user_id', 'screen_name'],
         require_auth = True
     )
@@ -418,14 +478,14 @@ class API(object):
     """ saved_searches """
     saved_searches = bind_api(
         path = '/saved_searches.json',
-        parser = parse_saved_searches,
+        payload_type = 'saved_search', payload_list = True,
         require_auth = True
     )
 
     """ saved_searches/show """
     get_saved_search = bind_api(
         path = '/saved_searches/show/{id}.json',
-        parser = parse_saved_search,
+        payload_type = 'saved_search',
         allowed_param = ['id'],
         require_auth = True
     )
@@ -434,7 +494,7 @@ class API(object):
     create_saved_search = bind_api(
         path = '/saved_searches/create.json',
         method = 'POST',
-        parser = parse_saved_search,
+        payload_type = 'saved_search',
         allowed_param = ['query'],
         require_auth = True
     )
@@ -443,7 +503,7 @@ class API(object):
     destroy_saved_search = bind_api(
         path = '/saved_searches/destroy/{id}.json',
         method = 'DELETE',
-        parser = parse_saved_search,
+        payload_type = 'saved_search',
         allowed_param = ['id'],
         require_auth = True
     )
@@ -451,18 +511,18 @@ class API(object):
     """ help/test """
     def test(self):
         try:
-            return bind_api(
+            bind_api(
                 path = '/help/test.json',
-                parser = parse_return_true
             )(self)
         except TweepError:
             return False
+        return True
 
     def create_list(self, *args, **kargs):
         return bind_api(
             path = '/%s/lists.json' % self.auth.get_username(),
             method = 'POST',
-            parser = parse_list,
+            payload_type = 'list',
             allowed_param = ['name', 'mode', 'description'],
             require_auth = True
         )(self, *args, **kargs)
@@ -471,7 +531,7 @@ class API(object):
         return bind_api(
             path = '/%s/lists/%s.json' % (self.auth.get_username(), slug),
             method = 'DELETE',
-            parser = parse_list,
+            payload_type = 'list',
             require_auth = True
         )(self)
 
@@ -479,41 +539,41 @@ class API(object):
         return bind_api(
             path = '/%s/lists/%s.json' % (self.auth.get_username(), slug),
             method = 'POST',
-            parser = parse_list,
+            payload_type = 'list',
             allowed_param = ['name', 'mode', 'description'],
             require_auth = True
         )(self, *args, **kargs)
 
     lists = bind_api(
         path = '/{user}/lists.json',
-        parser = parse_lists,
+        payload_type = 'list', payload_list = True,
         allowed_param = ['user', 'cursor'],
         require_auth = True
     )
 
     lists_memberships = bind_api(
         path = '/{user}/lists/memberships.json',
-        parser = parse_lists,
+        payload_type = 'list', payload_list = True,
         allowed_param = ['user', 'cursor'],
         require_auth = True
     )
 
     lists_subscriptions = bind_api(
         path = '/{user}/lists/subscriptions.json',
-        parser = parse_lists,
+        payload_type = 'list', payload_list = True,
         allowed_param = ['user', 'cursor'],
         require_auth = True
     )
 
     list_timeline = bind_api(
         path = '/{owner}/lists/{slug}/statuses.json',
-        parser = parse_statuses,
-        allowed_param = ['owner', 'slug', 'since_id', 'max_id', 'count', 'page']
+        payload_type = 'status', payload_list = True,
+        allowed_param = ['owner', 'slug', 'since_id', 'max_id', 'per_page', 'page']
     )
 
     get_list = bind_api(
         path = '/{owner}/lists/{slug}.json',
-        parser = parse_list,
+        payload_type = 'list',
         allowed_param = ['owner', 'slug']
     )
 
@@ -521,7 +581,7 @@ class API(object):
         return bind_api(
             path = '/%s/%s/members.json' % (self.auth.get_username(), slug),
             method = 'POST',
-            parser = parse_list,
+            payload_type = 'list',
             allowed_param = ['id'],
             require_auth = True
         )(self, *args, **kargs)
@@ -530,14 +590,14 @@ class API(object):
         return bind_api(
             path = '/%s/%s/members.json' % (self.auth.get_username(), slug),
             method = 'DELETE',
-            parser = parse_list,
+            payload_type = 'list',
             allowed_param = ['id'],
             require_auth = True
         )(self, *args, **kargs)
 
     list_members = bind_api(
         path = '/{owner}/{slug}/members.json',
-        parser = parse_users,
+        payload_type = 'user', payload_list = True,
         allowed_param = ['owner', 'slug', 'cursor']
     )
 
@@ -545,7 +605,7 @@ class API(object):
         try:
             return bind_api(
                 path = '/%s/%s/members/%s.json' % (owner, slug, user_id),
-                parser = parse_user
+                payload_type = 'user'
             )(self)
         except TweepError:
             return False
@@ -553,7 +613,7 @@ class API(object):
     subscribe_list = bind_api(
         path = '/{owner}/{slug}/subscribers.json',
         method = 'POST',
-        parser = parse_list,
+        payload_type = 'list',
         allowed_param = ['owner', 'slug'],
         require_auth = True
     )
@@ -561,14 +621,14 @@ class API(object):
     unsubscribe_list = bind_api(
         path = '/{owner}/{slug}/subscribers.json',
         method = 'DELETE',
-        parser = parse_list,
+        payload_type = 'list',
         allowed_param = ['owner', 'slug'],
         require_auth = True
     )
 
     list_subscribers = bind_api(
         path = '/{owner}/{slug}/subscribers.json',
-        parser = parse_users,
+        payload_type = 'user', payload_list = True,
         allowed_param = ['owner', 'slug', 'cursor']
     )
 
@@ -576,22 +636,22 @@ class API(object):
         try:
             return bind_api(
                 path = '/%s/%s/subscribers/%s.json' % (owner, slug, user_id),
-                parser = parse_user
+                payload_type = 'user'
             )(self)
         except TweepError:
             return False
 
-    """ trends/available [coming soon] """
+    """ trends/available """
     trends_available = bind_api(
         path = '/trends/available.json',
-        parser = parse_json,
+        payload_type = 'json',
         allowed_param = ['lat', 'long']
     )
 
-    """ trends/location [coming soon] """
+    """ trends/location """
     trends_location = bind_api(
         path = '/trends/{woeid}.json',
-        parser = parse_json,
+        payload_type = 'json',
         allowed_param = ['woeid']
     )
 
@@ -599,42 +659,67 @@ class API(object):
     search = bind_api(
         search_api = True,
         path = '/search.json',
-        parser = parse_search_results,
-        allowed_param = ['q', 'lang', 'locale', 'rpp', 'page', 'since_id', 'geocode', 'show_user']
+        payload_type = 'search_result', payload_list = True,
+        allowed_param = ['q', 'lang', 'locale', 'rpp', 'page', 'since_id', 'geocode', 'show_user', 'max_id', 'since', 'until', 'result_type']
     )
     search.pagination_mode = 'page'
 
     """ trends """
     trends = bind_api(
-        search_api = True,
         path = '/trends.json',
-        parser = parse_json
+        payload_type = 'json'
     )
 
     """ trends/current """
     trends_current = bind_api(
-        search_api = True,
         path = '/trends/current.json',
-        parser = parse_json,
+        payload_type = 'json',
         allowed_param = ['exclude']
     )
 
     """ trends/daily """
     trends_daily = bind_api(
-        search_api = True,
         path = '/trends/daily.json',
-        parser = parse_json,
+        payload_type = 'json',
         allowed_param = ['date', 'exclude']
     )
 
     """ trends/weekly """
     trends_weekly = bind_api(
-        search_api = True,
         path = '/trends/weekly.json',
-        parser = parse_json,
+        payload_type = 'json',
         allowed_param = ['date', 'exclude']
     )
 
+    """ geo/reverse_geocode """
+    reverse_geocode = bind_api(
+        path = '/geo/reverse_geocode.json',
+        payload_type = 'json',
+        allowed_param = ['lat', 'long', 'accuracy', 'granularity', 'max_results']
+    )
+
+    """ geo/nearby_places """
+    # listed as deprecated on twitter's API documents
+    nearby_places = bind_api(
+        path = '/geo/nearby_places.json',
+        payload_type = 'json',
+        allowed_param = ['lat', 'long', 'ip', 'accuracy', 'granularity', 'max_results']
+    )
+
+    """ geo/id """
+    geo_id = bind_api(
+        path = '/geo/id/{id}.json',
+        payload_type = 'json',
+        allowed_param = ['id']
+    )
+
+    """ geo/search """
+    geo_search = bind_api(
+        path = '/geo/search.json',
+        payload_type = 'json',
+        allowed_param = ['lat', 'long', 'query', 'ip', 'granularity', 'accuracy', 'max_results', 'contained_within']
+    )
+
     """ Internal use only """
     @staticmethod
     def _pack_image(filename, max_size):
@@ -643,7 +728,7 @@ class API(object):
         try:
             if os.path.getsize(filename) > (max_size * 1024):
                 raise TweepError('File is too big, must be less than 700kb.')
-        except os.error, e:
+        except os.error:
             raise TweepError('Unable to access file')
 
         # image must be gif, jpeg, or png
@@ -671,7 +756,7 @@ class API(object):
         # build headers
         headers = {
             'Content-Type': 'multipart/form-data; boundary=Tw3ePy',
-            'Content-Length': len(body)
+            'Content-Length': str(len(body))
         }
 
         return headers, body
